function Test(browser) {
    this._init(browser);
}

Test.prototype = {
    _init: function (browser) {
        this._url = browser.getUrl();
        this._browser = browser;
        browser.open(this._url);
    },

    onLoad: function () {
        setTimeout(this._browser.finish.bind(this._browser), 30000);
    },

    validateCallLog: function (log) {
        assertEquals('Unity.init', log[0].func, 'Unity.init');
        var i;
        var actionsCount = 0, launcherAction = 0;
        for (i = 0; i < log.length; i++) {
            if (log[i].func === 'Unity.Launcher.addAction') {
                launcherAction++;
            }
        }
        assertFalse('Unity.Launcher.addAction', !launcherAction);
    },

    scriptName: 'GoogleDocs.user.js'
};
